using System;
using System.Collections.Generic;
using System.Windows.Forms;
using EVEMon.Common;

namespace EVEMon.SkillPlanner
{
    public partial class SuggestionWindow : EVEMonForm
    {
        public SuggestionWindow()
        {
            InitializeComponent();
        }

        public SuggestionWindow(Plan p)
            : this()
        {
            m_plan = p;
        }

        private Plan m_plan;

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void SuggestionWindow_Load(object sender, EventArgs e)
        {
            IEnumerable<Plan.Entry> entries = m_plan.GetSuggestions();

            CharacterInfo gci = m_plan.GrandCharacterInfo;
            EveAttributeScratchpad scratchpad = new EveAttributeScratchpad();
            TimeSpan postTime = TimeSpan.Zero;

            lbSkills.Items.Clear();
            foreach (Plan.Entry pe in entries)
            {
                lbSkills.Items.Add(pe.SkillName + " " +
                                   Skill.GetRomanForInt(pe.Level));

                Skill gs = gci.GetSkill(pe.SkillName);
                postTime += gs.GetTrainingTimeOfLevelOnly(pe.Level, true, scratchpad);
                scratchpad.ApplyALevelOf(gs);
            }
            postTime += m_plan.GetTotalTime(scratchpad);
            TimeSpan preTime = m_plan.GetTotalTime(null);

            TimeSpan diff = preTime - postTime;

            lblBeforeTime.Text = Skill.TimeSpanToDescriptiveText(preTime, DTO_OPTS);
            lblAfterTime.Text = Skill.TimeSpanToDescriptiveText(postTime, DTO_OPTS);

            lblDiffTime.Text = Skill.TimeSpanToDescriptiveText(diff, DTO_OPTS);
        }

        private const DescriptiveTextOptions DTO_OPTS = DescriptiveTextOptions.IncludeCommas;
    }
}